function Q = IOC_MFADecoding( probs, y_C, sigma, CMtx )
% INPUT:
% probs: 1 * nLabel, estimated probability of labels (being 1)
% y_C: 1 * nCode, values of the predicted codes
% sigma: 1 * nCode, standard deviation of predicted codes
% CMtx: nLabel * nCode, the encoding matrix
%
% OUTPUT:
% y_hat: 1 * nLabel, decoded probability for each label

nLabel = length(probs);
nCode = length(y_C);
if length(sigma)~=nCode
    disp('length(sigma)~=nCode!');
    return;
end
if size(CMtx,1)~=nLabel
    disp('size(CMtx,1)~=nLabel!');
    return;
end
if size(CMtx,2)~=nCode
    disp('size(CMtx,2)~=nCode!');
    return;
end

%initialize as all negative labels
Q = probs;
unprocessed = ones(1, nLabel);
iter = 0;
i0 = 0;
%In each iteration, find one label to be processed
while nnz(unprocessed)>0
    iter = iter + 1;
    %find an unprocessed label
    unp_units = find(unprocessed); 
    if nnz(unp_units>i0)>0
        idx = find(unp_units>i0,1);
        i = unp_units(idx);
    else
        i = unp_units(1);
    end
    
    %update Q(i) using fixed point equation
    logPos = log(probs(i));
    logNeg = log(1-probs(i));
    
    %for k = 1 : nCode, updating together
    negFct = zeros(1,nCode);
    posFct = zeros(1,nCode);
    
    for j1 = 1 : nLabel
        % shortcut if j1==i OR Q(j1)==0
        if j1 == i || Q(j1) == 0
            continue;
        end
        
        %first-order terms that are not involving Q(i)
        negFct = negFct + (CMtx(j1,:)*Q(j1)).^2;
        negFct = negFct - 2*y_C.*CMtx(j1,:)*Q(j1);
        
        %second-order terms
        for j2 = 1 : (j1-1)
            % shortcut if j2 == i OR Q(j2) == 0
            if j2 == i || Q(j2) == 0
                continue;
            end
            
            %second-order terms
            negFct = negFct + 2*CMtx(j1,:).*CMtx(j2,:)*Q(j1)*Q(j2);
        end
        
        %terms involving Q(i) -- only updated for posFct
        posFct = posFct + 2*CMtx(i,:).*CMtx(j1,:)*Q(j1);
    end
    
    %adding terms without involving j1 and j2
    negFct = negFct + y_C.^2;
    posFct = posFct + CMtx(i,:).^2 - 2*y_C.*CMtx(i,:);
    posFct = posFct + negFct;
    
    %divide by sigma
    %negFct = negFct ./ (2*sigma.^2) + log(sigma) + 0.5*log(2*pi);
    %posFct = posFct ./ (2*sigma.^2) + log(sigma) + 0.5*log(2*pi);
    negFct = negFct ./ (2*sigma.^2);
    posFct = posFct ./ (2*sigma.^2);    


    logPos0 = logPos;
    logNeg0 = logNeg;
    %add them to logPos and losNeg
    logPos = logPos - sum(posFct);
    logNeg = logNeg - sum(negFct);
    
    oldQi = Q(i);
    Q(i) = exp(logPos) / (exp(logPos) + exp(logNeg));
    if abs(Q(i)-oldQi) > 0.0001
        %abs(Q(i)-oldQi)
        unprocessed = ones(1, nLabel);
    end
    unprocessed(i) = 0;
    i0 = i;
    
    %fprintf(1,'Iter: %g, i: %g, logPos: %g -> %g, logNeg: %g -> %g\n\tQ(i): %g --> %g, changing %g (probs(i): %g)\n', iter, i, logPos0, logPos, logNeg0, logNeg, oldQi, Q(i), abs(Q(i)-oldQi), probs(i));
end